package com.mytutorials.design_patterns.observer.subject.impl;

import java.util.ArrayList;
import java.util.List;

import com.mytutorials.design_patterns.observer.Observer;
import com.mytutorials.design_patterns.observer.subject.Subject;

public class StockGrabber implements Subject {

	private List<Observer> observers;

	private double ibmPrice;

	private double aaplPrice;

	private double googPrice;

	public StockGrabber() {

		// Creates an ArrayList to hold all observers
		observers = new ArrayList<Observer>();
	}

	public void register(Observer newObserver) {

		// Adds a new observer to the ArrayList
		observers.add(newObserver);
	}

	public void unregister(Observer deleteObserver) {

		// Get the index of the observer to delete
		int observerIndex = observers.indexOf(deleteObserver);

		// Print out message (Have to increment index to match)
		System.out.println("Observer " + (observerIndex + 1) + " deleted");

		// Removes observer from the ArrayList
		observers.remove(observerIndex);

	}

	public void notifyObserver() {

		// Cycle through all observers and notifies them of
		// price changes
		for (Observer observer : observers) {

			observer.update(ibmPrice, aaplPrice, googPrice);
		}
	}

	// Change prices for all stocks and notifies observers of changes

	public void setIBMPrice(double newIBMPrice) {

		this.ibmPrice = newIBMPrice;
		notifyObserver();

	}

	public void setAAPLPrice(double newAAPLPrice) {

		this.aaplPrice = newAAPLPrice;
		notifyObserver();

	}

	public void setGOOGPrice(double newGOOGPrice) {

		this.googPrice = newGOOGPrice;
		notifyObserver();

	}

}
