package com.mytutorials.design_patterns.builder.director;

import com.mytutorials.design_patterns.builder.api.RobotBuilder;
import com.mytutorials.design_patterns.builder.client.api.Robot;

//The director / engineer class creates a Robot using the
//builder interface that is defined (OldRobotBuilder)

public class RobotEngineer {

	private RobotBuilder robotBuilder;

	// OldRobotBuilder specification is sent to the engineer

	public RobotEngineer(RobotBuilder robotBuilder) {

		this.robotBuilder = robotBuilder;

	}

	// Return the Robot made from the OldRobotBuilder spec

	public Robot getRobot() {

		return this.robotBuilder.getRobot();

	}

	// Execute the methods specific to the RobotBuilder

	// that implements RobotBuilder (OldRobotBuilder)

	public void makeRobot() {

		this.robotBuilder.buildRobotHead();

		this.robotBuilder.buildRobotTorso();

		this.robotBuilder.buildRobotArms();

		this.robotBuilder.buildRobotLegs();

	}

}
